# Configuration File
_DEFAULT_SESSION = None  # Used to reuse generated session. DON'T CHANGE IT

USER_AGENT = "Barcelona 289.0.0.77.109 Android (31/12; 480dpi; 1080x2278; vivo; vivo 1939; 2004; qcom; en_US; 383877305)"

# request timeout - in seconds
TIMEOUT = 5

# Example {"http":"proxy_here","https":"proxy_here"} Accepts python dictionary.
PROXY = None

if __name__ == "__main__":
    pass
